@@ -259,6 +259,11 @@ impl<Dpy: DisplayBase> DisplayBase for RenderDisplay<Dpy> {
         self.inner.next_request_number()
     }
 
+    #[inline]
+    fn has_pending_event(&self) -> bool {
+        self.inner.has_pending_event()
+    }
+
     #[inline]
     fn push_event(&mut self, event: Event) {
         self.inner.push_event(event)
@@ -368,6 +373,11 @@ where
         self.inner().next_request_number()
     }
 
+    #[inline]
+    fn has_pending_event(&self) -> bool {
+        self.inner().has_pending_event()
+    }
+
     #[inline]
     fn push_event(&mut self, event: Event) {
         self.inner().push_event(event)
